#!/usr/bin/env python

# rgma-setup.py
#
# A script to set up an R-GMA client.
#

import os, sys, socket, getopt, re

def exit_bad_hostname(name):
    sys.stderr.write("\nERROR: %s is not a valid host name - please check\n" % name)
    sys.stderr.write("R-GMA configuration not completed\n")
    sys.exit(1)

def verify_hostname(name):
    regex = '^([\w-]+\.)*[\w-]+(:\d+)?$'
    return re.match(regex, name)
    
def version():

    print
    print("rgma-setup.py version 5.0.7")
    
def usage():

    version()
    print
    print("A tool to configure an R-GMA client.")
    print
    print("Usage: rgma-setup.py [ --help ] [ --version ]")
    print("                     [ --server=<url> ]")
    print("                     [ --schema=<url> ]")
    print("                     [ --registry=<url> ]")
    print("                     [ --secure=<true|false> ]")
    print("                     [ --silent ]")
    print("")
    print("Options:")
    print("  --server    Host name of the R-GMA server (default: local hostname)")
    print("  --schema    Host name of the R-GMA schema (default: same as server)")
    print("  --registry  Host name of the R-GMA registry (default: same as server)")
    print("  --secure    Whether to use the secure HTTPS protocol (default: true)")
    print("  --silent    Run non-interactively, using defaults for unspecified parameters")
    print("")
    print("To configure multiple registries, use the --registry option more than once.")
    print("")
    print("Host names may take an optional port number, e.g. example.com:8442. If the")
    print("port number is not present, 8443 is used by default if secure mode is enabled")
    print("or 8080 is used if secure mode is disabled.")
    
def yes_or_no(string):

    string = string.lower().strip()
    return string in ("y", "yes", "true")

def input_with_default(message, default):

    response = raw_input(message + " [" + default + "] ")
    if len(response.strip()) == 0:
        return default
    else:
        return response

def has_port(hostname):
    regex = re.compile("^.*:\d+$")
    return regex.match(hostname)
    
###########################
# Main script starts here #
###########################

# Check RGMA_HOME is set

try:
    rgma_home = os.environ['RGMA_HOME']
except KeyError:
    try:
        rgma_home = os.environ['GLITE_LOCATION']
    except KeyError:
        print "ERROR: Environment variable RGMA_HOME or GLITE_LOCATION must be set"
        sys.exit(1)
 
# Get hostname
 
hostname = socket.getfqdn();

# Parse command line

known_options = ["help", "version", \
                 "server=", "schema=","registry=", \
                 "secure=", "silent"]
try:
    options, args = getopt.getopt(sys.argv[1:], "", known_options)
except getopt.GetoptError:
    print("ERROR: parsing command line options\n")
    usage()
    sys.exit(1)

for arg in args:
    print("WARNING: unrecognised command line option '%s' ignored" % arg)

server = None
schema = None
registries = []
silent = False
secure = True

for option, value in options:
    if option == "--help":
        usage()
        sys.exit(0)
    elif option == "--version":
        version()
        sys.exit(0)
    elif option == "--schema":
        schema = value
    elif option == "--registry":
        registries.append(value)
    elif option == "--server":
        server = value
    elif option == "--silent":
        silent = True
    elif option == "--secure":
        secure = yes_or_no(value)

# Welcome

if not silent:
    print
    print "Welcome to the R-GMA setup utility"
    print "----------------------------------"
    print

# Configure rgma.conf

if server == None:
    if silent:
        server = hostname
    else:
        server = input_with_default("Enter the host name of the local R-GMA server", hostname)

if schema == None:
    if silent:
        schema = server
    else:
        schema = input_with_default("Enter the host name of the R-GMA schema", server)

if len(registries) == 0:
    if silent:
        registries.append(server)
    else:
        more = True
        default_registry = server

        while more:
            registry = input_with_default("Enter the host name of the R-GMA registry", default_registry)
            registry = registry.strip()
        
            if registry == "":
                print("Enter a valid hostname")
                more = True
            else:
                registries.append(registry)
                more = yes_or_no(raw_input("Configure more registry locations? (y/n) [n] "))
                default_registry = ""

server = server.strip()
schema = schema.strip()

if secure:
    protocol = "https"
    port = 8443
else:
    protocol = "http"
    port = 8080

if len(server) == 0:
  server = hostname
if len(schema) == 0:
  schema = hostname

# Check host names are valid

if not verify_hostname(server):
    exit_bad_hostname(server)

if not verify_hostname(schema):
    exit_bad_hostname(schema)

for registry in registries:
    registry = registry.strip()
    if not verify_hostname(registry):
        exit_bad_hostname(registry)

if has_port(server):
    server_url = "%s://%s/R-GMA/" % (protocol, server)
else:
    server_url = "%s://%s:%i/R-GMA/" % (protocol, server, port)

if has_port(schema):
    schema_url = "%s://%s/R-GMA/SchemaServlet" % (protocol, schema)
else:
    schema_url = "%s://%s:%i/R-GMA/SchemaServlet" % (protocol, schema, port)

# Write rgma.conf

conf_dir = os.path.join(rgma_home, "etc/rgma")
filename = os.path.join(conf_dir, "rgma.conf")

file = open(filename, "w")

file.write("Consumer=" + server_url + "ConsumerServlet\n")
file.write("PrimaryProducer=" + server_url + "PrimaryProducerServlet\n")
file.write("SecondaryProducer=" + server_url + "SecondaryProducerServlet\n")
file.write("OnDemandProducer=" + server_url + "OnDemandProducerServlet\n")

for registry in registries:
    registry = registry.strip()
    if has_port(registry):
        registry_url = "%s://%s/R-GMA/RegistryServlet" % (protocol, registry)
    else:
        registry_url = "%s://%s:%i/R-GMA/RegistryServlet" % (protocol, registry, port)
    file.write("Registry=" + registry_url + "\n")

file.write("Schema=" + schema_url + "\n")

file.close()

authfilename = os.path.join(conf_dir, "ClientAuthentication.props")

if not silent:
    print
    print "Configuration written to:"
    print
    print "  " + filename
    if (secure):
        print
        print "To configure security, edit proxy/certificate settings in"
        print
        print "  " + authfilename
    print
